#!/usr/bin/env python
# vim: set fileencoding=utf8:
"""
Unittest module of ...


AUTHOR:
    lambdalisue[Ali su ae] (lambdalisue@hashnote.net)
    
Copyright:
    Copyright 2011 Alisue allright reserved.

License:
    Licensed under the Apache License, Version 2.0 (the "License"); 
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unliss required by applicable law or agreed to in writing, software
    distributed under the License is distrubuted on an "AS IS" BASICS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""
__AUTHOR__ = "lambdalisue (lambdalisue@hashnote.net)"
from django.contrib.auth.models import User
from django.test import TestCase
from ..models import Entry

class EntryViewTestCase(TestCase):
    fixtures = ['test.yaml']

    def test_list_get(self):
        response = self.client.get('/')
        self.assertEqual(response.status_code, 200)

    def test_detail_get(self):
        # Anonymous user doesn't have view permission
        response = self.client.get('/foo/')
        self.assertRedirects(response, '/accounts/login/?next=/foo/')

        # Authenticated user has view permission
        assert self.client.login(username='foo', password='password')
        response = self.client.get('/foo/')
        self.assertEqual(response.status_code, 200)
        self.client.logout()

    def test_detail_get_invalid(self):
        # Permission doesn't affect object which doesn't exist
        response = self.client.get('/unknown/')
        self.assertEqual(response.status_code, 404)

    def test_create_get(self):
        response = self.client.get('/create/')
        # Anonymous user doesn't have create permission
        self.assertRedirects(response, '/accounts/login/?next=/create/')
    
        # Authenticated user has add permission
        assert self.client.login(username='foo', password='password')
        response = self.client.get('/create/')
        self.assertEqual(response.status_code, 200)
        self.client.logout()

    def test_create_post(self):
        # Anonymous user doesn't have add permission
        response = self.client.post('/create/', {
                'title': 'foobar', 'body': 'foobar'
            })
        self.assertRedirects(response, '/accounts/login/?next=/create/')
        
        # Authenticated user has add permission
        assert self.client.login(username='foo', password='password')
        response = self.client.post('/create/', {
                'title': 'foobar', 'body': 'foobar'
            })
        self.assertEqual(response.status_code, 302)
        assert Entry.objects.filter(title='foobar').exists()
        self.client.logout()

    def test_create_post_invalid(self):
        assert self.client.login(username='foo', password='password')
        response = self.client.post('/create/', {
                'title': '', 'body': ''
            })
        self.assertEqual(response.status_code, 200)
        assert 'This field is required' in response.content
        self.client.logout()

    def test_update_get(self):
        # the entry author is foo
        entry = Entry.objects.get(pk=2)
        foo = User.objects.get(username='foo')
        assert entry.author == foo
        assert foo.has_perm('blog.change_entry', entry)

        # Anonymous user doesn't have change permission
        response = self.client.get('/update/2/')
        self.assertRedirects(response, '/accounts/login/?next=/update/2/')

        # Non author user doesn't have change permission
        assert self.client.login(username='bar', password='password')
        response = self.client.get('/update/2/')
        self.assertEqual(response.status_code, 403)

        # Author has change permission
        assert self.client.login(username='foo', password='password')
        response = self.client.get('/update/2/')
        self.assertEqual(response.status_code, 200)

        # superuser has all permissions
        assert self.client.login(username='admin', password='password')
        response = self.client.get('/update/2/')
        self.assertEqual(response.status_code, 200)
        self.client.logout()

    def test_update_get_invalid(self):
        # Permission doesn't affect object which doesn't exist
        response = self.client.get('/update/999/')
        self.assertEqual(response.status_code, 404)

    def test_update_post(self):
        # the entry author is foo
        entry = Entry.objects.get(pk=2)
        foo = User.objects.get(username='foo')
        assert entry.author == foo
        assert foo.has_perm('blog.change_entry', entry)

        # Anonymous user doesn't have change permission
        response = self.client.post('/update/2/', {
                'title': 'foobar', 'body': 'foobar'
            })
        self.assertRedirects(response, '/accounts/login/?next=/update/2/')

        # Non author user doesn't have change permission
        assert self.client.login(username='bar', password='password')
        response = self.client.post('/update/2/', {
                'title': 'foobar', 'body': 'foobar'
            })
        self.assertEqual(response.status_code, 403)

        # Author user has change permission
        assert self.client.login(username='foo', password='password')
        response = self.client.post('/update/2/', {
                'title': 'foobar', 'body': 'foobar'
            })
        self.assertEqual(response.status_code, 302)
        self.assertEqual(Entry.objects.get(pk=2).title, 'foobar')
        self.assertEqual(Entry.objects.get(pk=2).body, 'foobar')

        # superuser has all permissions
        assert self.client.login(username='admin', password='password')
        response = self.client.post('/update/2/', {
                'title': 'foobar2', 'body': 'foobar2'
            })
        self.assertEqual(response.status_code, 302)
        self.assertEqual(Entry.objects.get(pk=2).title, 'foobar2')
        self.assertEqual(Entry.objects.get(pk=2).body, 'foobar2')
        self.client.logout()

    def test_update_post_invalid(self):
        # the entry author is foo
        entry = Entry.objects.get(pk=2)
        foo = User.objects.get(username='foo')
        assert entry.author == foo
        assert foo.has_perm('blog.change_entry', entry)

        assert self.client.login(username='foo', password='password')
        response = self.client.post('/update/2/', {
                'title': '', 'body': ''
            })
        self.assertEqual(response.status_code, 200)
        assert 'This field is required' in response.content
        self.client.logout()

    def test_delete_get(self):
        # the entry author is foo
        entry = Entry.objects.get(pk=2)
        foo = User.objects.get(username='foo')
        assert entry.author == foo
        assert foo.has_perm('blog.delete_entry', entry)

        # Anonymous user doesn't have delete permission
        response = self.client.get('/delete/2/')
        self.assertRedirects(response, '/accounts/login/?next=/delete/2/')

        # Non author user doesn't have delete permission
        assert self.client.login(username='bar', password='password')
        response = self.client.get('/delete/2/')
        self.assertEqual(response.status_code, 403)

        # Author user have delete permission
        assert self.client.login(username='foo', password='password')
        response = self.client.get('/delete/2/')
        self.assertEqual(response.status_code, 200)

        # superuser have all permissions
        assert self.client.login(username='admin', password='password')
        response = self.client.get('/delete/2/')
        self.assertEqual(response.status_code, 200)
        self.client.logout()

    def test_delete_get_invalid(self):
        response = self.client.get('/delete/999/')
        self.assertEqual(response.status_code, 404)

    def test_delete_post(self):
        # the entry author is foo
        entry = Entry.objects.get(pk=2)
        foo = User.objects.get(username='foo')
        assert entry.author == foo
        assert foo.has_perm('blog.delete_entry', entry)

        # Anonymous user doesn't have delete permission
        response = self.client.post('/delete/2/')
        self.assertRedirects(response, '/accounts/login/?next=/delete/2/')

        # Non author user doesn't have delete permission
        assert self.client.login(username='bar', password='password')
        response = self.client.post('/delete/2/')
        self.assertEqual(response.status_code, 403)

        # Author user has delete permission
        assert self.client.login(username='foo', password='password')
        response = self.client.post('/delete/2/')
        self.assertEqual(response.status_code, 302)
        assert not Entry.objects.filter(pk=2).exists()

        # superuser has all permissions
        assert self.client.login(username='admin', password='password')
        response = self.client.post('/delete/3/')
        self.assertEqual(response.status_code, 302)
        assert not Entry.objects.filter(pk=3).exists()
        self.client.logout()



#!/usr/bin/env python
# vim: set fileencoding=utf8:
"""
Base module of object-permission handler


AUTHOR:
    lambdalisue[Ali su ae] (lambdalisue@hashnote.net)
    
Copyright:
    Copyright 2011 Alisue allright reserved.

License:
    Licensed under the Apache License, Version 2.0 (the "License"); 
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unliss required by applicable law or agreed to in writing, software
    distributed under the License is distrubuted on an "AS IS" BASICS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""
__AUTHOR__ = "lambdalisue (lambdalisue@hashnote.net)"
from django.db.models.signals import post_save
from django.db.models.signals import post_delete

from observer import watch

from ..mediators import ObjectPermMediator

class ObjectPermHandlerBase(ObjectPermMediator):
    """Base class of ObjectPermHandler
    
    ObjectPermHandler is a subclass of ObjectPermMediator so the class has all
    attributes/methods of the ObjectPermMediator

    Bind the model class and ObjectPermHandler then automatically 'setup' and 
    'teardown' method of the class is called when the instance of the model is
    created/deleted.

    watch attrs in 'setup' method then automatically 'updated' method is called
    when the watched attr is updated.
    """
    def __init__(self, instance):
        super(ObjectPermHandlerBase, self).__init__(instance)
        self.model = instance.__class__

    @classmethod
    def bind(cls, model):
        """bind model and this handler"""
        post_save.connect(cls._post_save_reciever, sender=model, weak=False)
        post_delete.connect(cls._post_delete_reciver, sender=model, weak=False)

    @classmethod
    def unbind(cls, model):
        """unbind model and this handler"""
        post_save.disconnect(cls._post_save_reciever, sender=model)
        post_delete.disconnect(cls._post_delete_reciver, sender=model)

    @classmethod
    def _post_save_reciever(cls, sender, instance, created, **kwargs):
        # register the instance to this class
        if created or cls.get(instance) is None:
            cls._register(instance)

    @classmethod
    def _post_delete_reciver(cls, sender, instance, **kwargs):
        # unregister the instance from this class
        cls._unregister(instance)

    @classmethod
    def _register(cls, instance):
        # create new handler instance
        self = cls(instance)
        # register the handler instance to cls
        if not hasattr(cls, '_handlers'):
            cls._handlers = {}
        cls._handlers[instance] = self
        # call pre setup method
        self._setup()
        # call pre updated method
        self._updated(attr=None)

    @classmethod
    def _unregister(cls, instance):
        # if no handler is registered, just ignore
        if not hasattr(cls, '_handlers'):
            return
        self = cls._handlers[instance]
        # call pre teardown method
        self._teardown()
        del cls._handlers[instance]

    @classmethod
    def get(cls, instance):
        if not hasattr(cls, '_handlers'):
            cls._handlers = {}
        return cls._handlers.get(instance)

    @classmethod
    def update(cls):
        """call 'updated' method of all instance of this class"""
        if not hasattr(cls, '_handlers'):
            return
        for model, handler in cls._handlers.iteritems():
            handler._updated(attr=None)

    def _setup(self):
        raise NotImplementedError
    def _teardown(self):
        raise NotImplementedError
    def _updated(self):
        raise NotImplementedError

class ObjectPermHandler(ObjectPermHandlerBase):
    def __init__(self, instance):
        super(ObjectPermHandler, self).__init__(instance)
        self._watchers = {}

    def _watch_update_reciver(self, sender, obj, attr):
        # update instance (because self.instance is not fresh)
        self.instance = obj
        # call pre updated method
        self._updated(attr)

    def _watch(self, instance, attr, callback):
        """Watch instance attr and call callback and register the watcher"""
        if instance not in self._watchers:
            self._watchers[instance] = []
        watcher = watch(instance, attr, callback)
        self._watchers[instance].append(watcher)

    def _unwatch(self, instance=None):
        """Unwatch watchers of instance."""
        if instance:
            if instance in self._watchers:
                del self._watchers[instance]
        else:
            # unwatch all watchers registered in this class instance
            self._watchers = {}

    def watch(self, attr, instance=None):
        """Watch instance attr and call 'updated' method of this class"""
        if instance is None:
            instance = self.instance
        self._watch(instance ,attr, self._watch_update_reciver)

    def _setup(self):
        """pre setup method"""
        self.setup()

    def _teardown(self):
        """pre teardown method"""
        # unwatch all watchers registered in this class instance
        self._unwatch(instance=None)
        self.teardown()

    def _updated(self, attr):
        """pre updated method"""
        # reset object permission of the instance
        self.reset()
        # call updated method
        self.updated(attr)

    def setup(self):
        """called when the bind model instance is created."""
        pass

    def updated(self, attr):
        """called when watched attr of the instance is updated"""
        raise NotImplementedError

    def teardown(self):
        """called when the bind model instance is deleted."""
        pass



#!/usr/bin/env python
# vim: set fileencoding=utf8:
"""
Object permission handler for model of testing app


AUTHOR:
    lambdalisue[Ali su ae] (lambdalisue@hashnote.net)
    
Copyright:
    Copyright 2011 Alisue allright reserved.

License:
    Licensed under the Apache License, Version 2.0 (the "License"); 
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unliss required by applicable law or agreed to in writing, software
    distributed under the License is distrubuted on an "AS IS" BASICS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""
__AUTHOR__ = "lambdalisue (lambdalisue@hashnote.net)"
from object_permission import site
from object_permission.handlers import ObjectPermHandler

from models import Article

class ArticleObjectPermHandler(ObjectPermHandler):
    def setup(self):
        # Watch related fields with observer
        self.watch('pub_state')
        self.watch('author')
        self.watch('inspectors')

    def updated(self, attr):
        # Author has manager permission
        self.manager(self.instance.author)
        if self.instance.pub_state == 'draft':
            # No-one can view the object
            self.reject(self.instance.inspectors)
            self.reject(None)
            self.reject('anonymous')
        else:
            # Inspector has editor permission
            self.editor(self.instance.inspectors)
            # Authenticated user has viewer permission
            self.viewer(None)
            if self.instance.pub_state == 'inspecting':
                # Anonymous user has no permissions
                self.reject('anonymous')
            else:
                # Article published
                self.viewer('anonymous')
# Register
site.register(Article, ArticleObjectPermHandler)



#!/usr/bin/env python
# vim: set fileencoding=utf8:
"""
Unittest module of ...


AUTHOR:
    lambdalisue[Ali su ae] (lambdalisue@hashnote.net)
    
Copyright:
    Copyright 2011 Alisue allright reserved.

License:
    Licensed under the Apache License, Version 2.0 (the "License"); 
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unliss required by applicable law or agreed to in writing, software
    distributed under the License is distrubuted on an "AS IS" BASICS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""
__AUTHOR__ = "lambdalisue (lambdalisue@hashnote.net)"
from django.contrib.auth.models import User
from django.contrib.auth.models import AnonymousUser

from base import ObjectPermissionTestCaseBase

class ArticleTestCase(ObjectPermissionTestCaseBase):
    def setUp(self):
        from app.models import Article
        self.foo = User.objects.create_user(
                username='foo',
                email='foo@test.com',
                password='password')
        self.foofoo = User.objects.create_user(
                username='foofoo',
                email='foofoo@test.com',
                password='password')
        self.foofoofoo = User.objects.create_user(
                username='foofoofoo',
                email='foofoofoo@test.com',
                password='password')
        self.bar = User.objects.create_user(
                username='bar',
                email='bar@test.com',
                password='password')
        self.barbar = User.objects.create_user(
                username='barbar',
                email='barbar@test.com',
                password='password')
        self.hoge = AnonymousUser()

        self.article = Article.objects.create(author=self.foo)
        self.article.inspectors.add(self.foofoo, self.foofoofoo)
        self.assertEqual(self.article.author, self.foo)

    def test_permissions_created(self):
        # pub_state = draft
        # Author
        self.assert_(self.foo.has_perm('app.view_article', self.article))
        self.assert_(self.foo.has_perm('app.change_article', self.article))
        self.assert_(self.foo.has_perm('app.delete_article', self.article))
        # Inspector
        self.assert_(not self.foofoo.has_perm('app.view_article', self.article))
        self.assert_(not self.foofoo.has_perm('app.change_article', self.article))
        self.assert_(not self.foofoo.has_perm('app.delete_article', self.article))
        # Authenticated user
        self.assert_(not self.bar.has_perm('app.view_article', self.article))
        self.assert_(not self.bar.has_perm('app.change_article', self.article))
        self.assert_(not self.bar.has_perm('app.delete_article', self.article))
        # Anonymous user
        self.assert_(not self.hoge.has_perm('app.view_article', self.article))
        self.assert_(not self.hoge.has_perm('app.change_article', self.article))
        self.assert_(not self.hoge.has_perm('app.delete_article', self.article))

    def test_permission_watched_pub_state_updated(self):
        # pub_state = inspecting
        self.article.pub_state = 'inspecting'
        self.article.save()
        # Author
        self.assert_(self.foo.has_perm('app.view_article', self.article))
        self.assert_(self.foo.has_perm('app.change_article', self.article))
        self.assert_(self.foo.has_perm('app.delete_article', self.article))
        # Inspector
        self.assert_(self.foofoo.has_perm('app.view_article', self.article))
        self.assert_(self.foofoo.has_perm('app.change_article', self.article))
        self.assert_(not self.foofoo.has_perm('app.delete_article', self.article))
        # Authenticated user
        self.assert_(self.bar.has_perm('app.view_article', self.article))
        self.assert_(not self.bar.has_perm('app.change_article', self.article))
        self.assert_(not self.bar.has_perm('app.delete_article', self.article))
        # Anonymous user
        self.assert_(not self.hoge.has_perm('app.view_article', self.article))
        self.assert_(not self.hoge.has_perm('app.change_article', self.article))
        self.assert_(not self.hoge.has_perm('app.delete_article', self.article))

        # pub_state = published
        self.article.pub_state = 'published'
        self.article.save()
        # Author
        self.assert_(self.foo.has_perm('app.view_article', self.article))
        self.assert_(self.foo.has_perm('app.change_article', self.article))
        self.assert_(self.foo.has_perm('app.delete_article', self.article))
        # Inspector
        self.assert_(self.foofoo.has_perm('app.view_article', self.article))
        self.assert_(self.foofoo.has_perm('app.change_article', self.article))
        self.assert_(not self.foofoo.has_perm('app.delete_article', self.article))
        # Authenticated user
        self.assert_(self.bar.has_perm('app.view_article', self.article))
        self.assert_(not self.bar.has_perm('app.change_article', self.article))
        self.assert_(not self.bar.has_perm('app.delete_article', self.article))
        # Anonymous user
        self.assert_(self.hoge.has_perm('app.view_article', self.article))
        self.assert_(not self.hoge.has_perm('app.change_article', self.article))
        self.assert_(not self.hoge.has_perm('app.delete_article', self.article))

        # pub_state = draft
        self.article.pub_state = 'draft'
        self.article.save()
        # Author
        self.assert_(self.foo.has_perm('app.view_article', self.article))
        self.assert_(self.foo.has_perm('app.change_article', self.article))
        self.assert_(self.foo.has_perm('app.delete_article', self.article))
        # Inspector
        self.assert_(not self.foofoo.has_perm('app.view_article', self.article))
        self.assert_(not self.foofoo.has_perm('app.change_article', self.article))
        self.assert_(not self.foofoo.has_perm('app.delete_article', self.article))
        # Authenticated user
        self.assert_(not self.bar.has_perm('app.view_article', self.article))
        self.assert_(not self.bar.has_perm('app.change_article', self.article))
        self.assert_(not self.bar.has_perm('app.delete_article', self.article))
        # Anonymous user
        self.assert_(not self.hoge.has_perm('app.view_article', self.article))
        self.assert_(not self.hoge.has_perm('app.change_article', self.article))
        self.assert_(not self.hoge.has_perm('app.delete_article', self.article))



#!/usr/bin/env python
# vim: set fileencoding=utf8:
"""
TestCase module for testing App

Original code is found at
http://stackoverflow.com/questions/502916/django-how-to-create-a-model-dynamically-just-for-testing
"""
import copy

from django.conf import settings
from django.core.management import call_command
from django.db.models import loading
from django import test

class AppTestCase(test.TestCase):
    installed_apps = []

    def _get_installed_apps(self):
        _installed_apps = list(settings.INSTALLED_APPS)
        for app in self.installed_apps:
            if app not in _installed_apps:
                _installed_apps.append(app)
        return _installed_apps

    def _pre_setup(self):
        # store original installed apps
        self._original_installed_apps = list(settings.INSTALLED_APPS)
        # get extra required apps
        settings.INSTALLED_APPS = self._get_installed_apps()
        # Call syncdb to create db for extra apps (migrate=False for South)
        loading.cache.loaded = False
        call_command('syncdb', interactive=False, verbosity=0, migrate=False)
        # Call the original method that does the fixtures etc.
        super(AppTestCase, self)._pre_setup()

    def _post_teardown(self):
        # Call the original method
        super(AppTestCase, self)._post_teardown()
        # Restore the settings
        settings.INSTALLED_APPS = self._original_installed_apps
        loading.cache.loaded = False



#!/usr/bin/env python
# vim: set fileencoding=utf8:
"""
short module explanation


AUTHOR:
    lambdalisue[Ali su ae] (lambdalisue@hashnote.net)
    
Copyright:
    Copyright 2011 Alisue allright reserved.

License:
    Licensed under the Apache License, Version 2.0 (the "License"); 
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unliss required by applicable law or agreed to in writing, software
    distributed under the License is distrubuted on an "AS IS" BASICS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""
__AUTHOR__ = "lambdalisue (lambdalisue@hashnote.net)"
from django.conf import settings
from app_test import AppTestCase

class ObjectPermissionTestCaseBase(AppTestCase):
    installed_apps = [
            'author',
            'object_permission.tests.app',
        ]
    middleware_classes = [
            'object_permission.backends.ObjectPermBackend',
            'author.middlewares.AuthorDefaultBackendMiddleware',
        ]
    def _pre_setup(self):
        self._original_middleware_classes = settings.MIDDLEWARE_CLASSES
        _middleware_classes = list(settings.MIDDLEWARE_CLASSES)
        for middleware_class in self.middleware_classes:
            if middleware_class not in _middleware_classes:
                _middleware_classes.append(middleware_class)
        settings.MIDDLEWARE_CLASSES = _middleware_classes
        super(ObjectPermissionTestCaseBase, self)._pre_setup()
        # run autodiscover
        from .. import autodiscover
        autodiscover()
    def _post_teardown(self):
        super(ObjectPermissionTestCaseBase, self)._post_teardown()
        settings.MIDDLEWARE_CLASSES = self._original_middleware_classes
        



#!/usr/bin/env python
# vim: set fileencoding=utf8:
"""
Object permission handler for model which has author field


AUTHOR:
    lambdalisue[Ali su ae] (lambdalisue@hashnote.net)
    
Copyright:
    Copyright 2011 Alisue allright reserved.

License:
    Licensed under the Apache License, Version 2.0 (the "License"); 
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unliss required by applicable law or agreed to in writing, software
    distributed under the License is distrubuted on an "AS IS" BASICS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""
__AUTHOR__ = "lambdalisue (lambdalisue@hashnote.net)"
from base import ObjectPermHandler

class AuthorObjectPermHandler(ObjectPermHandler):
    """ObjectPermHandler for model which has author field

    This handler contribute..

        1.  Manager permission to instance author
        2.  Viewer permission to authenticated user
        3.  Viewer permission to anonymous user if reject_anonymous is False

    """
    author_field = 'author'
    reject_anonymous = False

    def get_author(self):
        """get author field value"""
        return getattr(self.instance, self.author_field)

    def setup(self):
        # watch author field
        self.watch(self.author_field)

    def updated(self, attr):
        # Author has full access
        self.manager(self.get_author())
        # Authenticated user can view
        self.viewer(None)
        if self.reject_anonymous:
            self.reject('anonymous')
        else:
            self.viewer('anonymous')
